@@ -12,6 +12,7 @@ from .verify_database import main as verify_database_main
 from .resource_pack.main import main as resource_pack_main
 from .manual.main import main as manual_main
 from .datapack.main import main as datapack_main
+from .compatibilities.main import main as compatibilities_main
 from .finalyze import main as finalyze_main
 
 # Functions
@@ -83,7 +84,7 @@ def build_process(config: dict, setup_database: callable, setup_external_databas
 	config = enhance_config_main(config)
 
 	# Get start time & Enable colors in Windows 10 console
-	START_TIME = time.perf_counter()
+	START_TIME: float = time.perf_counter()
 	os.system("color")
 	info("Starting build process...")
 
@@ -107,10 +108,13 @@ def build_process(config: dict, setup_database: callable, setup_external_databas
 	# Generate datapack
 	datapack_main(config)
 
+	# Special compatibilities with featured datapacks
+	compatibilities_main(config)
+
 	# Finalyze build process
 	finalyze_main(config, user_code)
 
 	# Total time
-	total_time = time.perf_counter() - START_TIME
-	info(f"Build finished in {total_time:.3f} seconds")
+	TOTAL_TIME: float = time.perf_counter() - START_TIME
+	info(f"Build finished in {TOTAL_TIME:.5f} seconds")
 
