@@ -9,6 +9,7 @@ from ..utils.print import *
 
 def main(config: dict):
 	print()
+	start_time: float = time.perf_counter()
 
 	# Generate datapack loading
 	loading_main(config)
@@ -24,5 +25,8 @@ def main(config: dict):
 
 	# Generate items loot tables
 	loot_tables_main(config)
-	info("Datapack successfully generated!")
+
+	# Info print
+	total_time: float = time.perf_counter() - start_time
+	info(f"Datapack successfully generated in {total_time:.5f} seconds!")
 
