@@ -114,11 +114,11 @@ class EquipmentsConfig():
 		return {key: value for key, value in self.attributes.items() if key not in NOT_ON_ARMOR}
 
 # UUIDs utils
-def format_attributes(attributes: str, slot: str, config: dict = {}) -> list[dict]:
+def format_attributes(config: dict, attributes: str, slot: str, attr_config: dict = {}) -> list[dict]:
 	""" Returns generated attribute_modifiers key for an item (adds up attributes and config) """
 	# Get attributes from config
 	attribute_modifiers = []
-	for attribute_name, value in config.items():
+	for attribute_name, value in attr_config.items():
 		if attribute_name != "durability":
 			attribute_modifiers.append({"type": attribute_name, "amount": value, "operation": "add_value", "slot": slot, "id": f"{config['namespace']}:{attribute_name}.{slot}"})
 
@@ -166,7 +166,7 @@ def generate_everything_about_this_ore(config: dict, database: dict[str, dict],
 		color = [sum(x) / len(color) for x in zip(*color)]					# Get the average color
 		color = int(color[0]) << 16 | int(color[1]) << 8 | int(color[2])	# Convert to int (Minecraft format: Red<<16 + Green<<8 + Blue)
 
-	# Placeables (ore, block, raw_block)
+	# Placeables (ore, deepslate_ore, block, raw_block)
 	for block in [f"{material_base}_block", f"{material_base}_ore", f"deepslate_{material_base}_ore", f"raw_{material_base}_block"]:
 		if block + ".png" not in config['textures_files']:
 			continue
@@ -219,7 +219,7 @@ def generate_everything_about_this_ore(config: dict, database: dict[str, dict],
 		if item.endswith("nugget"):
 			for gear in SLOTS.keys():
 				if f"{material_base}_{gear}.png" in config['textures_files']:
-					database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"equipment","ingredient":ingr_repr(f"{material_base}_{gear}", ns = config['namespace'])})
+					database[item][RESULT_OF_CRAFTING].append({"type":"smelting","result_count":1,"category":"equipment","experience":0.8,"cookingtime":100,"ingredient":ingr_repr(f"{material_base}_{gear}", ns = config['namespace'])})
 		if item.endswith("stick"):
 			database[item][RESULT_OF_CRAFTING].append({"type":"crafting_shaped","result_count":4,"category":"misc","shape":["X","X"],"ingredients":{"X":main_ingredient}})
 		if item.endswith("rod"):
@@ -255,7 +255,7 @@ def generate_everything_about_this_ore(config: dict, database: dict[str, dict],
 			database[armor][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["X X","X X"],"ingredients":{"X": main_ingredient}}]
 			gear_config = VanillaEquipments.BOOTS.value[equivalent_to]
 			database[armor]["max_damage"] = int(gear_config["durability"] * durability_factor)
-		database[armor]["attribute_modifiers"] = format_attributes(armor_attributes, SLOTS[gear], gear_config)
+		database[armor]["attribute_modifiers"] = format_attributes(config, armor_attributes, SLOTS[gear], gear_config)
 	
 	# Tools (sword, pickaxe, axe, shovel, hoe)
 	for gear in ["sword", "pickaxe", "axe", "shovel", "hoe"]:
@@ -290,11 +290,11 @@ def generate_everything_about_this_ore(config: dict, database: dict[str, dict],
 			gear_config = VanillaEquipments.HOE.value[equivalent_to]
 			database[tool]["max_damage"] = int(gear_config["durability"] * durability_factor)
 			database[tool][RESULT_OF_CRAFTING] = [{"type":"crafting_shaped","result_count":1,"category":"equipment","shape":["XX"," S"," S"],"ingredients": tools_ingr}]
-		database[tool]["attribute_modifiers"] = format_attributes(tools_attributes, SLOTS[gear], gear_config)
+		database[tool]["attribute_modifiers"] = format_attributes(config, tools_attributes, SLOTS[gear], gear_config)
 	pass
 
 # Generate everything about these ores
-def generate_everything_about_these_ores(database: dict[str, dict], ores: dict[str, EquipmentsConfig|None]) -> dict[str, list[str]]:
+def generate_everything_about_these_ores(config: dict, database: dict[str, dict], ores: dict[str, EquipmentsConfig|None]) -> dict[str, list[str]]:
 	""" Uses function 'generate_everything_about_this_ore' for each ore in the ores dictionary.
 	Args:
 		database	(dict[str, dict]):	The database to apply the ores to.
@@ -303,8 +303,8 @@ def generate_everything_about_these_ores(database: dict[str, dict], ores: dict[s
 		dict[str, list[str]]:	The list of generated items for each ore.
 	"""
 	generated_items = {}
-	for material, config in ores.items():
-		generated_items[material] = generate_everything_about_this_ore(database, material, config)
+	for material, ore_config in ores.items():
+		generated_items[material] = generate_everything_about_this_ore(config, database, material, ore_config)
 	return generated_items
 
 
@@ -348,11 +348,12 @@ def generate_custom_records(config: dict, database: dict[str, dict], records: di
 
 
 # Deterministic custom model data
-def deterministic_custom_model_data(config: dict, database: dict[str, dict], starting_cmd: int) -> None:
+def deterministic_custom_model_data(config: dict, database: dict[str, dict], starting_cmd: int, black_list: list[str] = []) -> None:
 	""" Apply custom model data to all items using a cache method.
 	Args:
 		database		(dict[str, dict]):	The database to apply custom model data to.
 		starting_cmd	(int):				The starting custom model data.
+		blacklist		(list[str]):		The list of items to ignore.
 	"""
 	# Load cached custom model data
 	cached_custom_model_data = {}
@@ -373,7 +374,7 @@ def deterministic_custom_model_data(config: dict, database: dict[str, dict], sta
 	
 	# For each item in the database, apply its new custom model data if it doesn't exist
 	for item, data in database.items():
-		if not data.get("custom_model_data") or not isinstance(data["custom_model_data"], int):
+		if item not in black_list and (not data.get("custom_model_data") or not isinstance(data["custom_model_data"], int)):
 			data["custom_model_data"] = max_cmd + 1
 
 			# Add two custom model data if the item has an on/off texture
@@ -384,7 +385,8 @@ def deterministic_custom_model_data(config: dict, database: dict[str, dict], sta
 	
 	# Update cache
 	for item, data in database.items():
-		cached_custom_model_data[item] = data["custom_model_data"]
+		if data.get("custom_model_data"):
+			cached_custom_model_data[item] = data["custom_model_data"]
 	with open(config['cmd_cache'], "w") as f:
 		super_json_dump(cached_custom_model_data, f)
 	return
@@ -454,3 +456,118 @@ def add_smithed_ignore_vanilla_behaviours_convention(database: dict[str, dict])
 			data["custom_data"]["smithed"]["ignore"]["crafting"] = True
 	return
 
+class CustomOreGeneration():
+	def __init__(self, dimensions: list[str], maximum_height: int = 70, minimum_height: int|None = None, veins_per_region: int = 4, vein_size_logic: float = 0.4, provider: str|list[str] = "#minecraft:overworld_carver_replaceables", placer_command: str = "") -> None:
+		""" Creates a custom ore generation configuration.\n
+		Args:
+			dimensions (list[str]):			The dimensions to generate the ore in (ex: ["minecraft:overworld"])
+			maximum_height (int):			The maximum height the ore can generate at
+			minimum_height (int|None):		The minimum height the ore can generate at (default: None for bedrock to maximum_height)
+			veins_per_region (int):			The number of veins per region
+			vein_size_logic (float):		The logic to calculate the vein size, higher = more ores
+			provider (str|list[str]):		The block or block tag to check for the ore to generate (default is same as vanilla ores in overworld), can be either a string or a list of strings
+			placer_command (str):			The placer command to use for the ore generation, usually a function to place the custom ore but can be anything, default is autogenerated
+		"""
+		self.dimensions: list[str] = dimensions
+		self.maximum_height: int = maximum_height
+		self.minimum_height: int|None = minimum_height
+		self.veins_per_region: int = veins_per_region
+		self.vein_size_logic: float = vein_size_logic
+		self.provider: str|list[str] = provider
+		self.placer_command: str = placer_command
+		if not official_lib_used("smart_ore_generation"):
+			debug("Found custom ore generation, adding 'smart_ore_generation' dependency")
+		self.check_validity()
+	
+	def check_validity(self) -> list[str]:
+		""" Check if the custom ore generation configuration is valid """
+		if not isinstance(self.dimensions, list) or not all(isinstance(dimension, str) for dimension in self.dimensions):
+			error("Custom ore generation 'dimensions' must be a list of strings")
+		if not isinstance(self.maximum_height, int):
+			error("Custom ore generation 'maximum_height' must be an integer")
+		if self.minimum_height is not None and not isinstance(self.minimum_height, int):
+			error("Custom ore generation 'minimum_height' must be an integer or None")
+		elif self.minimum_height is not None and self.minimum_height > self.maximum_height:
+			error("Custom ore generation 'minimum_height' must be less or equal to 'maximum_height'")
+		if not isinstance(self.veins_per_region, int):
+			error("Custom ore generation 'veins_per_region' must be an integer")
+		if not isinstance(self.vein_size_logic, float) or self.vein_size_logic < 0:
+			error("Custom ore generation 'vein_size_logic' must be a float >= 0")
+		if not isinstance(self.provider, str) and not isinstance(self.provider, list):
+			error("Custom ore generation 'provider' must be a string or a list of strings")
+		if isinstance(self.provider, list) and not all(isinstance(provider, str) for provider in self.provider):
+			error("Custom ore generation 'provider' must be a list of strings")
+		if not isinstance(self.placer_command, str):
+			error("Custom ore generation 'placer_command' must be a string")
+	
+	def generate_files(self, config: dict, custom_ore: str, number: int|None = None):
+		""" Generate the files for the custom ore generation\n
+		Args:
+			config (dict):			The configuration dictionary
+			custom_ore (str):		The custom ore to generate, ex: "adamantium_ore"
+			number (int|None):		(optional) The number of the custom ore to generate, ex: 1 for "adamantium_ore_1", used when having multiple configurations for the same ore
+		"""
+		namespace: str = config['namespace']
+		database: dict[str, dict] = config['database']
+		lib_folder: str = f"{config['build_datapack']}/data/{config['namespace']}/function/calls/smart_ore_generation"
+		beautify_ore: str = custom_ore.replace("_", " ").title()
+
+		# Setup placer command if not done yet
+		if not self.placer_command:
+			self.placer_command = f"run function {namespace}:custom_blocks/{custom_ore}/place_main"
+
+			# If the custom ore is a deepslate ore and normal ore exists, cancel the command if the block is not stone
+			if custom_ore.startswith("deepslate_") and custom_ore.replace("deepslate_", "") in database:
+				self.placer_command = "unless block ~ ~ ~ minecraft:stone " + self.placer_command
+
+			# If the custom ore is a normal ore and deepslate ore exists, cancel the command if the block is not deepslate
+			elif f"deepslate_{custom_ore}" in database:
+				self.placer_command = "unless block ~ ~ ~ minecraft:deepslate " + self.placer_command
+		
+		# Add ore to generate ores function
+		path: str = f"{lib_folder}/generate_ores.mcfunction"
+		dimensions_check: str = ""
+		for index, dimension in enumerate(self.dimensions):
+			dimensions_check += f"\nexecute if dimension {dimension} run scoreboard players set #dimension smart_ore_generation.data {index}"
+		if self.minimum_height == None:
+			op_mini: str = f"scoreboard players operation #min_height smart_ore_generation.data = _OVERWORLD_BOTTOM smart_ore_generation.data"
+		else:
+			op_mini: str = f"scoreboard players set #min_height smart_ore_generation.data {self.minimum_height}"
+		content: str = f"""
+# Generate {beautify_ore} (x{self.veins_per_region})
+scoreboard players set #dimension smart_ore_generation.data -1{dimensions_check}
+{op_mini}
+scoreboard players set #max_height smart_ore_generation.data {self.maximum_height}
+"""
+		place_vein: str = f"execute if score #dimension smart_ore_generation.data matches 0.. run function {config['namespace']}:calls/smart_ore_generation/veins/{custom_ore}\n"
+		content += self.veins_per_region * place_vein
+		write_to_file(path, content)
+
+		# Add ore to veins folder
+		path: str = f"{lib_folder}/veins/{custom_ore}.mcfunction"
+		content: str = f"""
+# Try to find a random position adjacent to air in the region to generate the ore
+function #smart_ore_generation:v1/slots/random_position
+
+# Placing {beautify_ore} patch
+execute at @s if block ~ ~ ~ {self.provider} {self.placer_command}
+"""
+		if self.vein_size_logic > 0:
+			radius: float = self.vein_size_logic % 1
+			while radius <= self.vein_size_logic:
+				
+				# Make a sphere of radius
+				for x in range(-1, 2):
+					for y in range(-1, 2):
+						for z in range(-1, 2):
+							content += f"execute at @s positioned ~{x*radius} ~{y*radius} ~{z*radius} if block ~ ~ ~ {self.provider} {self.placer_command}\n"
+				
+				# Increase radius
+				radius += 1
+		
+		# Write file
+		write_to_file(path, content)
+		
+
+
+
